/*
 * AugmentedGrammarRule.java
 *
 * George Ferguson, ferguson@cs.rochester.edu, 14 May 2002
 * Time-stamp: <Fri May 23 12:36:48 EDT 2003 ferguson>
 */

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.NoSuchElementException;

/**
 * An AugmentedGrammarRule is one whose constituents are AugmentedConstituents
 * (ie., which have features associated with them).
 */
public class AugmentedGrammarRule extends GrammarRule {
    /**
     * Creates a new AugmentedGrammarRule.
     * Note that we don't check that the rhs actually consists only
     * of AugmentedConstituents (but one could do that).
     */
    public AugmentedGrammarRule(Symbol id, AugmentedConstituent lhs, List rhs) {
	super(id, lhs, rhs);
    }
    /**
     * Returns a copy of this AugmentedGrammarRule with the given
     * bindings applied.
     */
    public AugmentedGrammarRule copyWithBindings(BindingSet bindings) {
	Debug.debug("AugmentedGrammarRule.copy: " + this);
	Debug.debug("    bindings = " + bindings);
	AugmentedConstituent lhs = (AugmentedConstituent)getLhs();
	Debug.debug("AugmentedGrammarRule.copy: copying lhs " + lhs);
	AugmentedConstituent newlhs = (AugmentedConstituent)lhs.copyWithBindings(bindings);
	List newrhs = new LinkedList();
	Iterator constits = getRhs().iterator();
	try {
	    while (constits.hasNext()) {
		AugmentedConstituent constit = (AugmentedConstituent)constits.next();
		Debug.debug("AugmentedGrammarRule.copy: copying rhs constit " + constit);
		newrhs.add(constit.copyWithBindings(bindings));
	    }
	} catch (NoSuchElementException ex) {
	}
	Debug.debug("AugmentedGrammarRule.copy: done");
	return new AugmentedGrammarRule(getId(), newlhs, newrhs);
    }
}
