/*
 * Arc.java
 *
 * George Ferguson, ferguson@cs.rochester.edu, 13 May 2002
 * Time-stamp: <Wed May 14 12:00:05 EDT 2003 ferguson>
 */

/**
 * An Arc represents an active arc on the chart.
 */
public class Arc {
    /**
     * The rule this Arc is tracking.
     */
    GrammarRule rule;
    /**
     * The starting index of this Arc.
     */
    int start;
    /**
     * The ending index of this Arc.
     */
    int end;
    /**
     * The index of the next rhs constituent wanted by this Arc.
     */
    int next;
    /**
     * The length of the rhs of the rule associated with this Arc.
     */
    int rhsLength;
    /**
     * Creates a new Arc with the given parameters.
     */
    public Arc(GrammarRule rule, int start, int end, int next) {
	this.rule = rule;
	this.start=start;
	this.end = end;
	this.next = next;
	// Cache for later testing of arc completion
	this.rhsLength = rule.getRhs().size();
    }
    /**
     * Returns the rule this Arc is tracking.
     */
    public GrammarRule getRule() {
	return rule;
    }
    /**
     * Returns the starting index of this Arc.
     */
    public int getStart() {
	return start;
    }
    /**
     * Returns the ending index of this Arc.
     */
    public int getEnd() {
	return end;
    }
    /**
     * Returns the index of the next rhs constituent wanted by this Arc.
     */
    public int getNext() {
	return next;
    }
    /**
     * Returns the next rhs constituent wanted by this Arc.
     */
    public Constituent getNextConstituent() {
	return (Constituent)rule.getRhs().get(next);
    }
    /**
     * Returns true if this Arc is complete.
     */
    public boolean isComplete() {
	return (next >= rhsLength);
    }
    /**
     * Returns the String representation of this Arc.
     */
    public String toString() {
	return rule.toString(next) + " from " + start + " to " + end;
    }
}
