/*
 * Chart.java
 *
 * George Ferguson, ferguson@cs.rochester.edu, 13 May 2002
 * Time-stamp: <Wed May 14 12:02:08 EDT 2003 ferguson>
 */

import java.util.List;
import java.util.LinkedList;

/**
 * The basic data structure for a chart parser, constisting of a set of
 * entries (completed constituents) and a set of active arcs (representing
 * partially-completed constituents corresponding to grammar rules).
 */
public class Chart {
    /**
     * Entries (completed constituents) indexed by word position
     * (ie., each position contains a list of all the constituents
     * starting at that position).
     */
    List[] entries;
    /**
     * Active arcs indexed by word position (ie., each position
     * contains a list of all the active arcs ENDING at that
     * position).
     */
    List[] arcs;
    /**
     * Create a new Chart of the given length.
     */
    public Chart(int length) {
	entries = new List[length];
	for (int i=0; i < length; i++) {
	    entries[i] = new LinkedList();
	}
	// Add one for arcs that end at (after) the end of the sentence
	arcs = new List[length+1];
	for (int i=0; i < length+1; i++) {
	    arcs[i] = new LinkedList();
	}
    }
    /**
     * Adds given Entry (completed constituent) to this Chart,
     * indexed by starting position.
     */
    public void add(Entry entry) {
	entries[entry.getStart()].add(entry);
    }
    /**
     * Adds given Arc (active arc) to this Chart,
     * indexed by ending index.
     */
    public void add(Arc arc) {
	arcs[arc.getEnd()].add(arc);
    }
    /**
     * Returns the list of Arcs that end at the given position.
     */
    public List getArcsEndingAtPosition(int end) {
	return arcs[end];
    }
}
