/*
 * Lexicon.java
 *
 * George Ferguson, ferguson@cs.rochester.edu, 25 Apr 2002
 * Time-stamp: <Thu May 22 15:21:44 EDT 2003 ferguson>
 */

import java.io.IOException;
import java.io.PrintStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.NoSuchElementException;

/**
 * A Lexicon is a set of Constituents indexed by word. There can be more than
 * one Constituent for a word (corresponding to different senses of the word).
 */
public class Lexicon extends HashMap {
    /**
     * Add Constituent to list of entries for word, creating the list
     * if necessary.
     */
    public void add(Symbol word, Constituent constit) {
	// Get list of entries for this word
	List entries = (List)get(word);
	// Create list if necessary
	if (entries == null) {
	    entries = new LinkedList();
	    put(word, entries);
	}
	// Add constit to list of entries for this word
	entries.add(constit);
    }
    /**
     * Retrieve the List of Constituents associated with WORD.
     */
    public List getEntriesForWord(Symbol word) {
	List result = (List)get(word);
	if (result == null) {
	    result = new LinkedList();
	    put(word, result);
	}
	return result;
    }
    /**
     * Dump the contents of this Lexicon to System.out.
     */
    public void dump() {
	dump(System.out);
    }
    /**
     * Dump the contents of this Lexicon to the given PrintStream.
     */
    public void dump(PrintStream stream) {
	stream.println(description);
	Iterator keys = keySet().iterator();
	try {
	    while (keys.hasNext()) {
		Symbol word = (Symbol)keys.next();
		Iterator entries = getEntriesForWord(word).iterator();
		while (entries.hasNext()) {
		    Constituent constit = (Constituent)entries.next();
		    stream.println(word + ": " + constit.toString());
		}
	    }
	} catch (NoSuchElementException ex) {
	}
    }
    /**
     * The text description of this Lexicon. This may be useful in
     * displays, tools, or just as a comment in the files.
     */
    String description;
    /**
     * Returns the text description of this Lexicon.
     */
    public String getDescription() {
	return description;
    }
    /**
     * Sets the text description of this Lexicon.
     */
    public void setDescription(String description) {
	this.description = description;
    }
    /**
     * Read the given filename and augment this Lexicon.
     * @see LexiconFileHandler
     */
    public void readFile(String filename) throws IOException {
	new LexiconFileHandler(this).readFile(filename);
    }
}
