/*
 * HeadFeaturedGrammarRule.java
 *
 * George Ferguson, ferguson@cs.rochester.edu, 14 May 2002
 * Time-stamp: <Wed May 14 12:39:28 EDT 2003 ferguson>
 */

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.NoSuchElementException;

/**
 * An HeadFeaturedGrammarRule is one whose constituents are
 * HeadFeaturedConstituents (meaning that they could be marked
 * as head constituent).
 */
public class HeadFeaturedGrammarRule extends AugmentedGrammarRule {
    /**
     * Creates a new HeadFeaturedGrammarRule.
     * Note that we don't check that the rhs actually consists only
     * of HeadFeaturedConstituents (but one could do that).
     * <p>
     * Note that we apply the head feature conventions here in the
     * constructor rather than later to minimize code duplication with
     * parent classes.
     */
    public HeadFeaturedGrammarRule(Symbol id, HeadFeaturedConstituent lhs, List rhs) {
	super(id, lhs, rhs);
    }
    /**
     * Returns the head constituent of the rhs of this GrammarRule.
     */
    public HeadFeaturedConstituent getHeadConstituent() {
	Iterator constits = getRhs().iterator();
	try {
	    while (constits.hasNext()) {
		HeadFeaturedConstituent constit = (HeadFeaturedConstituent)constits.next();
		if (constit.isHead()) {
		    return constit;
		}
	    }
	} catch (NoSuchElementException ex) {
	}
	return null;
    }
}
