/*
 * BindingSet.java
 *
 * George Ferguson, ferguson@cs.rochester.edu, 24 Apr 2002
 * Time-stamp: <Wed May 14 12:19:10 EDT 2003 ferguson>
 */

import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * A BindingSet is a set of Bindings.
 * More specifically, it is a set of variables and their associated
 * bindings (for example, there will/should be at most one binding for
 * any var in the BindingSet. This allows certain optimizations.
 * The traditional Lisp implementation is an alist.
 */
class BindingSet extends AssociationList {
    /**
     * Creates a new empty BidningSet.
     */
    public BindingSet() {
    }
    /**
     * Sets the binding of the given Variable in this BindingSet to the
     * given value.
     */
    public void setBinding(Variable var, Object value) {
	put(var, value);
    }
    /**
     * Returns the binding of the given Variable in this BindingSet (or
     * null if there is none).
     */
    public Object getBinding(Variable var) {
	return get(var);
    }
    /**
     * Returns the String representation of this BindingSet.
     */
    public String toString() {
	// Don't know why this didn't work exactly... (bogus java polymorphism)
	// return Printing.listToString(entrySet().iterator());
	Iterator iterator = entrySet().iterator();
	StringBuffer buf = new StringBuffer("(");
	try {
	    while (iterator.hasNext()) {
		OrderedPair binding = (OrderedPair)iterator.next();
		buf.append(binding.toString());
		if (iterator.hasNext()) {
		    buf.append(" ");
		}
	    }
	} catch (NoSuchElementException ex) {
	}
	buf.append(")");
	return buf.toString();
    }
}
