/*
 * OrderedPair.java
 *
 * George Ferguson, ferguson@cs.rochester.edu, 17 Apr 2002
 * Time-stamp: <Wed May 14 12:16:09 EDT 2003 ferguson>
 */

import java.util.Map;

/**
 * An OrderedPair is a key/value pair. It implements the Map.Entry
 * interface and is returned by AssociationList.entrySet().
 */
public class OrderedPair implements Map.Entry {
    /**
     * The key of this OrderedPair.
     */
    Object key;
    /**
     * The value of this OrderedPair.
     */
    Object value;
    /**
     * Returns a new OrderedPair with the given key and value.
     */
    public OrderedPair(Object key, Object value) {
	this.key = key;
	this.value = value;
    }
    /**
     * Tests the given Object for equality with this OrderedPair.
     * Per the Map.Entry interface convention, teh following must hold
     * for this method to return true:
     * (1) the given Object must itself be an OrderedPair,
     * (2) both keys must either be null or equals() must be true of them, and
     * (3) both values must either be null or equals() must be true of them.
     */
    public boolean equals(Object o) {
	if (!(o instanceof OrderedPair)) {
	    return false;
	}
	OrderedPair opair = (OrderedPair)o;
	return ((key == null ?
		 opair.key == null : key.equals(opair.key))  &&
		(value == null ?
		 opair.value ==null : value.equals(opair.value)));
    }
    /**
     * Returns the key of this OrderedPair.
     */
    public Object getKey() {
	return key;
    }
    /**
     * Returns the value of this OrderedPair.
     */
    public Object getValue() {
	return value;
    }
    /**
     * Returns the hashCode of this OrderedPair.
     * Per the Map.Entry interface convention, this is computed as the
     * logical XOR (``^'') of the key and value hashCodes (using 0 for
     * null key or value).
     */
    public int hashCode() {
	return ((key == null ? 0 : key.hashCode()) ^
		(value == null ? 0 : value.hashCode()));
    }
    /**
     * Sets the value of this OrdredPair.
     * Per the Map.Entry interface, this method returns the previous
     * value.
     */
    public Object setValue(Object value) {
	Object oldvalue = this.value;
	this.value = value;
	return oldvalue;
    }
    /**
     * Returns the String representation of this OrderedPair.
     */
    public String toString() {
	return "(" + key.toString() + " . " + value.toString() + ")";
    }
}
